/* xml-test.cc */
/* Created by Enomoto Sanshiro on 30 July 2003. */
/* Last updated by Enomoto Sanshiro on 30 July 2003. */


#include <iostream>
#include <string>
#include "MushXml.hh"

using namespace mush;


class XmlFileViewer: public sax::DocumentHandler {
  public:
    XmlFileViewer(void);
    virtual ~XmlFileViewer();
    virtual void go(const std::string& file_name) throw(sax::SAXException);
  public:
    virtual void endDocument(void);
    virtual void startElement(const std::string& name, const TMushSaxAttributeList& attribute_list);
    virtual void endElement(const std::string& name);
    virtual void characters(const std::string& text);
  protected:
    sax::Parser* _parser;
};



XmlFileViewer::XmlFileViewer(void)
{
    _parser = new sax::Parser;
    _parser->setDocumentHandler(this);
}

XmlFileViewer::~XmlFileViewer()
{
    delete _parser;
}

void XmlFileViewer::go(const std::string& file_name) throw(sax::SAXException)
{
    _parser->parse(file_name);
}

void XmlFileViewer::endDocument(void)
{
    std::cout << std::endl;
}

void XmlFileViewer::startElement(const std::string& name, const TMushSaxAttributeList& attribute_list)
{
    std::cout << "<" << name;

    for (unsigned i = 0; i < attribute_list.getLength(); i++) {
	std::cout << " " << attribute_list.getName(i);
	std::cout << "=\"" << attribute_list.getValue(i) << "\"";
    }

    std::cout << ">";
}

void XmlFileViewer::endElement(const std::string& name)
{
    std::cout << "</" << name << ">";
}

void XmlFileViewer::characters(const std::string& text)
{
    std::cout << text;
}


int main(int argc, char** argv)
{
    if (argc < 2) {
	std::cerr << "Usage: " << argv[0] << " file_name" << std::endl;
	return -1;
    }

    XmlFileViewer xml_file_viewer;

    try {
	xml_file_viewer.go(argv[1]);
    }
    catch (sax::SAXException &e) {
	std::cerr << "ERROR: " << e << std::endl;
    }

    return 0;
}
