/* KinokoCanvasDrawableObject.hh */
/* Created by Enomoto Sanshiro on 11 July 2000. */
/* Last updated by Enomoto Sanshiro on 12 June 2004. */


#ifndef __KinokoCanvasDrawableObject_hh
#define __KinokoCanvasDrawableObject_hh


#include <iostream>
#include <string>
#include <deque>
#include "KinokoCanvasMarker.hh"
#include "KinokoCanvasTypesetter.hh"
#include "KinokoCanvasObject.hh"


class TKinokoCanvasDrawableObject: public TKinokoCanvasObject {
  public:
    TKinokoCanvasDrawableObject(TKinokoCanvas* Canvas, TKinokoCanvasImageArea* ImageArea);
    virtual ~TKinokoCanvasDrawableObject();
  protected:
    virtual int ProcessCommand(const std::string& Command, std::istream& InputStream);
    virtual int ProcessSetCommand(const std::string& Name, const std::string& Value);
    virtual int ProcessDrawPointCommand(std::istream& InputStream);
    virtual int ProcessDrawLineCommand(std::istream& InputStream);
    virtual int ProcessDrawRectCommand(std::istream& InputStream, bool IsFilled);
    virtual int ProcessDrawPolygonCommand(std::istream& InputStream, bool IsFilled);
    virtual int ProcessDrawCircleCommand(std::istream& InputStream, bool IsFilled);
    virtual int ProcessDrawEllipseCommand(std::istream& InputStream, bool IsFilled);
    virtual int ProcessDrawArrowTipCommand(std::istream& InputStream, bool IsFilled);
    virtual int ProcessDrawTextCommand(std::istream& InputStream);
    virtual int ProcessDrawImageCommand(std::istream& InputStream);
    virtual int ProcessClearCommand(std::istream& InputStream);
    virtual int ProcessSetFontCommand(std::istream& InputStream);
    virtual int ProcessSaveContextCommand(std::istream& InputStream);
    virtual int ProcessRestoreContextCommand(std::istream& InputStream);
  protected:
    virtual float CanvasXOf(float x);
    virtual float CanvasYOf(float y);
    virtual int SetCoordinate(float XMin, float YMin, float XWidth, float YWidth);
    virtual int SaveContext(void);
    virtual int RestoreContext(void);
  protected:
    int SetColor(const std::string& ColorName);
    int SetLineWidth(const std::string& LineWidthString);
    int SetLineStyle(const std::string& LineStyleString);
    int SetFont(const std::string& FontName, int FontSize);
    int SetTextAdjustment(const std::string& AdjustmentString);
    int SetMarker(std::string MarkerName);
    int SetMarkerSize(const std::string& MarkerSizeString);
  protected:
    int _ColorIndex;
    int _LineWidth;
    int _LineStyleIndex;
    int _FontIndex;
    TKinokoCanvasMarker *_Marker;
    int _MarkerSize;
    std::string _TextAdjustment;
    float _XMin, _XMax, _YMin, _YMax, _XWidth, _YWidth;
    TKinokoCanvasMarkerFactory *_MarkerFactory;
    TKinokoCanvasMathTextComposer* _MathTextComposer;
  private:
    std::deque<int> _ColorIndexStack;
    std::deque<int> _LineWidthStack;
    std::deque<int> _LineStyleIndexStack;
    std::deque<int> _FontIndexStack;
    std::deque<std::string> _TextAdjustmentStack;
    std::deque<TKinokoCanvasMarker*> _MarkerStack;
    std::deque<int> _MarkerSizeStack;
};


#endif
