/* KinokoCanvasPlotObject.hh */
/* Created by Enomoto Sanshiro on 11 July 2000. */
/* Last updated by Enomoto Sanshiro on 12 June 2004. */


#ifndef __KinokoCanvasPlotObject_hh
#define __KinokoCanvasPlotObject_hh


#include <string>
#include <iostream>
#include "KinokoCanvasDataScale.hh"
#include "KinokoCanvasColorScale.hh"
#include "KinokoCanvasFramedObject.hh"


class TKinokoCanvasPlotObject: virtual public TKinokoCanvasFramedObject {
  public:
    TKinokoCanvasPlotObject(TKinokoCanvas* Canvas, TKinokoCanvasImageArea* ImageArea);
    virtual ~TKinokoCanvasPlotObject();
    virtual TKinokoShellObject* Clone(void);
    virtual std::vector<std::string> ActionList(void);
    virtual int ProcessAction(const std::string& ActionName);
  protected:
    virtual int ProcessCommand(const std::string& Command, std::istream& InputStream);
    virtual int ProcessSetCommand(const std::string& Name, const std::string& Value);
    virtual int ProcessPlotCommand(std::istream& InputStream, int Style = 0);
    virtual int ProcessErrorPlotCommand(std::istream& InputStream);
    virtual int ProcessMinmaxPlotCommand(std::istream& InputStream);
    virtual int ProcessHistCommand(std::istream& InputStream, int Style = 0);
    virtual int ProcessCommentCommand(std::istream& InputStream);
    virtual int ProcessDrawXScaleLabelCommand(std::istream& InputStream);
    virtual int ProcessDrawYScaleLabelCommand(std::istream& InputStream);
  protected:
    virtual float CanvasXOf(float x);
    virtual float CanvasYOf(float y);
    virtual int SetCoordinate(float XMin, float YMin, float XWidth, float YWidth);
    virtual int SetXScaleLinear(void);
    virtual int SetXScaleLog(void);
    virtual int SetYScaleLinear(void);
    virtual int SetYScaleLog(void);
  protected:
    virtual int DrawFrame(void);
  protected:
    TKinokoCanvasDataScale *_XScale, *_YScale;
    int _NumberOfXScaleLabels, _NumberOfYScaleLabels;
    bool _IsXScaleLabelDisabled, _IsYScaleLabelDisabled;
    bool _IsXScaleLog, _IsYScaleLog;
    float _LogXMin, _LogXMax, _LogYMin, _LogYMax;
    float _LogXWidth, _LogYWidth;
    float _XMinOrg, _XWidthOrg, _YMinOrg, _YWidthOrg;
    std::string _XTitle, _YTitle;
    bool _IsYTitleVertical;
  private:
    enum TStyle {
	Style_None = 0x0000,
	Style_Points = 0x0001,
	Style_Lines = 0x0002,
	Style_Filled = 0x0004
    };
};


class TKinokoCanvasMapPlotObject: public TKinokoCanvasColorScaledObject {
  public:
    TKinokoCanvasMapPlotObject(TKinokoCanvas* Canvas, TKinokoCanvasImageArea* ImageArea);
    virtual ~TKinokoCanvasMapPlotObject();
    virtual TKinokoShellObject* Clone(void);
  protected:
    virtual float CanvasXOf(float x);
    virtual float CanvasYOf(float y);
    virtual int ProcessCommand(const std::string& Command, std::istream& InputStream);
    virtual int ProcessPlotCommand(std::istream& InputStream);    
  protected:
    virtual int DrawFrame(void);
};


#endif

