/* KinokoMonitor.cc */
/* Created by Enomoto Sanshiro on 16 May 2004. */
/* Last updated by Enomoto Sanshiro on 16 May 2003. */


#include <string>
#include <sstream>
#include "MushMisc.hh"
#include "KinokoMonitor.hh"

using namespace std;



TKinokoMonitorComponentCondition::TKinokoMonitorComponentCondition(const std::string& Name)
{
    _Name = Name;

    _CheckTime = 0;
    _State = "UNKNOWN";
    _HeartBeatCount = 0;
}

TKinokoMonitorComponentCondition::~TKinokoMonitorComponentCondition()
{
}

void TKinokoMonitorComponentCondition::Update(const std::string& CheckTime, const std::string& State, const std::string& HeartBeatCount)
{
    {
	istringstream is(CheckTime);
	is >> _CheckTime;
    }

    _State = State;

    {
	istringstream is(HeartBeatCount);
	is >> _HeartBeatCount;
    }
}



TKinokoMonitor::TKinokoMonitor(void)
{
    _Parser = new TMushSaxParser();
    _Parser->setDocumentHandler(this);

    _UpdateTime = TMushDateTime::SecSinceEpoch();
}

TKinokoMonitor::~TKinokoMonitor()
{
    delete _Parser;

    map<string, TKinokoMonitorComponentCondition*>::iterator i;
    for (i = _ComponentList.begin(); i != _ComponentList.end(); i++) {
	delete i->second;
    }
}

void TKinokoMonitor::Update(const std::string& RegistryFileName)
{
    try {
	_IsIteratingPropertyList = false;
	_CurrentComponentName = "";

	_Parser->parse(RegistryFileName);

	_UpdateTime = TMushDateTime::SecSinceEpoch();
    }
    catch (TSystemCallException &e) {
	;
    }
}

void TKinokoMonitor::startElement(const std::string& Name, const TMushSaxAttributeList& AttributeList)
{
    if (! _IsIteratingPropertyList) {
	_IsIteratingPropertyList = (Name == "__properties");
    }
    else if (_CurrentComponentName.empty()) {
	_CurrentComponentName = Name;
    }
    else {
	if (Name == "condition_check_time") {
	    _CheckTime = AttributeList.getValue("value");
	}
	else if (Name == "state") {
	    _State = AttributeList.getValue("value");
	}
	else if (Name == "heart_beat_count") {
	    _HeartBeatCount = AttributeList.getValue("value");
	}
    }
}

void TKinokoMonitor::endElement(const std::string& Name) 
{
    if (_IsIteratingPropertyList) {
	if (Name == _CurrentComponentName) {
	    TKinokoMonitorComponentCondition* Component;
	    if (_ComponentList.count(Name) == 0) {
		Component = new TKinokoMonitorComponentCondition(Name);
		_ComponentList[Name] = Component;
		_ComponentNameList.push_back(Name);
	    }
	    else {
		Component = _ComponentList[Name];
	    }
	    Component->Update(_CheckTime, _State, _HeartBeatCount);

	    _CurrentComponentName = "";
	}
	else if (Name == "__properties") {
	    _IsIteratingPropertyList = false;
	}
    }
}
